==> projectaria_client_sdk-1.0.0-cp39-cp39-macosx_10_16_x86_64.whl: projectaria_client_sdk-1.0.0.data/purelib/aria/doctor.py <==
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import hashlib
import logging
import os
import platform
import re
import stat
import subprocess
import sys

import tempfile
from dataclasses import dataclass
from typing import Callable, List, Optional

from termcolor import colored


def running_as_root() -> bool:
    return os.getuid() == 0


def run_command(cmd: List[str]) -> subprocess.CompletedProcess:
    logging.debug("Running: %s", " ".join(cmd))
    return subprocess.run(cmd, text=True, capture_output=True)


def failure(message: str):
    print(f"[ fail] {message}")
    exit(1)


def success(message: str):
    pill = colored("[ pass]", "white", "on_green", attrs=["bold"])
    message = colored(message, "green")
    print(pill, message)


def warning(message: str):
    pill = colored("[ warn]", "black", "on_yellow", attrs=["bold"])
    message = colored(message, "yellow")
    print(pill, message)


def action(message: str):
    print(f"    [+] {message}")


@dataclass
class Command:
    description: str
    cmd: List[str]


@dataclass
class File:
    description: str
    platforms: List[str]
    name: str
    path: str
    contents: str
    hash_suffix: bool = False
    permissions: Optional[int] = None
    owner: Optional[str] = None
    post_command: Optional[Command] = None

    def hash(self) -> str:
        return hashlib.sha1(self.contents.encode("utf8")).hexdigest()

    def file_path(self) -> str:
        if self.hash_suffix:
            name = f"{self.name}-{self.hash()}"
        else:
            name = self.name
        return os.path.join(self.path, name)

    def _read(self) -> str:
        with open(self.file_path(), "r") as f:
            return f.read()

    def _is_correct(self) -> bool:
        filepath = self.file_path()
        if not os.path.exists(filepath):
            return False
        # If we hash the file, no need to read contents
        if self.hash_suffix:
            return True
        if self.permissions is not None:
            st = os.stat(filepath)
            observed_permissions = stat.S_IMODE(st[stat.ST_MODE])
            if observed_permissions != self.permissions:
                logging.debug(
                    "Expected mode %s, got %s",
                    oct(self.permissions),
                    oct(observed_permissions),
                )
                return False
        try:
            d = self._read()
            if d != self.contents:
                logging.debug("File `%s` contents are outdated", filepath)
                return False
        except PermissionError:
            warning(f"Coud not check {self.description} file, please re-run as sudo")
        return True

    def ensure(
        self,
    ):
        if platform.system() not in self.platforms:
            return

        filepath = self.file_path()

        def remediation():
            if not os.path.exists(self.path):
                failure(
                    f"Path `{self.path}` not found, please use a supported Linux distribution"
                )

            action(f"Writing {self.description} at {filepath}")

            if self.owner == "root":
                # Remove any files with outdated hashes
                if self.hash_suffix:
                    for file in os.listdir(self.path):
                        if os.path.isfile(
                            os.path.join(self.path, file)
                        ) and file.startswith(self.name + "-"):
                            action(f"Removing old {self.description} file {file}")
                            run_command(["sudo", "rm", os.path.join(self.path, file)])
                # Write file and copy to correct location
                with tempfile.NamedTemporaryFile("w") as f:
                    f.write(self.contents)
                    f.flush()
                    run_command(["sudo", "cp", f.name, filepath])
                    if self.permissions is not None:
                        run_command(
                            ["sudo", "chmod", oct(self.permissions)[2:], filepath]
                        )
            else:
                with open(filepath, "w") as f:
                    f.write(self.contents)
                if self.permissions is not None:
                    os.chmod(filepath, self.permissions)

            if self.post_command is not None:
                action(self.post_command.description)
                run_command(self.post_command.cmd)

        if not self._is_correct():
            remediate(
                f"{self.description} is not configured correctly.",
                "update the configuration",
                remediation,
            )

        success(f"{self.description} ok")


REQUIRED_CONFIGURATION_FILES = [
    File(
        description="Aria udev rules",
        platforms=["Linux"],
        name="52-aria.rules",
        path="/etc/udev/rules.d",
        owner="root",
        permissions=0o644,
        post_command=Command(
            description="Reloading udev rules",
            cmd=["sudo", "udevadm", "control", "—reload-rules"],
        ),
        contents="""
SUBSYSTEM=="usb", ATTRS{idVendor}=="2833", MODE="0666"
SUBSYSTEM=="net", ACTION=="add", ATTRS{idVendor}=="2833" ATTRS{idProduct}=="0300", NAME="aria"
""",
    ),
    File(
        description="Aria network manager connection",
        platforms=["Linux"],
        name="aria",
        path="/etc/NetworkManager/system-connections",
        permissions=0o600,
        hash_suffix=True,
        owner="root",
        post_command=Command(
            description="Reloading network manager connections",
            cmd=["sudo", "nmcli", "connection", "reload"],
        ),
        contents="""
[connection]
id=Aria
interface-name=aria
autoconnect=true
type=ethernet

[ipv4]
method=auto
never-default=true

[ipv6]
method=auto
never-default=true
""",
    ),
]


def remediate(message: str, action: str, remediation: Callable):
    print(f"[error] {message}")
    input_message = f"        Would you like to {action}? ([y]/n): "
    ans = input(input_message)
    if ans == "" or ans == "y":
        try:
            remediation()
        except Exception as e:
            logging.debug("Exception: %s", e)
            failure(
                f"Failed to {action}. Please check https://facebookresearch.github.io/projectaria_tools/docs/ARK/sdk/troubleshoot"
            )

    else:
        exit(1)


def check_glib() -> bool:
    FAILED_TO_VALIDATE = "Failed validate glibc version, please check you are using a supported Linux distribution"

    if platform.system() != "Linux":
        return True

    output = run_command(["ldd", "--version", "ldd"])

    if output.returncode != 0:
        failure(FAILED_TO_VALIDATE)

    m = re.search(r"ldd .+ (\d+).(\d+)", output.stdout)

    if not m:
        failure(FAILED_TO_VALIDATE)

    ldd_major = int(m.group(1))
    ldd_minor = int(m.group(2))

    if ldd_major != 2 or ldd_minor < 34:
        failure(
            f"Your system glibc ({ldd_major}.{ldd_minor}) is too old, at least v2.34 is required. Please use a more recent Linux distribution"
        )

    success("glibc version ok")


def check_python_version():
    ver = sys.version_info
    if ver.minor == 8 and ver.micro < 10:
        failure(
            "Python versions below 3.8.10 are not supported. Please update your python version."
        )
    success("Python version ok")


def check_macos_network():
    """
    On MacOS, check that the `Aria` network service is last in the service order.
    This ensures that Aria USB streaming does not prevent internet access
    via other services (e.g. Wi-Fi or ethernet)
    """

    if platform.system() != "Darwin":
        return

    response = run_command(["networksetup", "-listnetworkserviceorder"])

    if response.returncode != 0:
        failure("Failed to list network interfaces.")

    interfaces = [
        match.group(1)
        for match in re.finditer(r"^\(\d+\) (.+)$", response.stdout, flags=re.MULTILINE)
    ]

    ARIA_INTERFACE = "Aria"

    if ARIA_INTERFACE not in interfaces:
        warning(
            "No Aria network interface found, this will be created after you first start streaming. "
            "If you lose internet access when starting streaming over USB, please run `aria-doctor` "
            "again to restore connectivity."
        )
        return

    def remediation():
        # Move the Aria interface to lowest priority
        interfaces.remove(ARIA_INTERFACE)
        interfaces.append(ARIA_INTERFACE)

        response = run_command(
            ["sudo", "networksetup", "-ordernetworkservices"] + interfaces
        )

        if response.returncode != 0:
            logging.debug(
                "networksetup failed: %i, %d", response.returncode, response.stdout
            )
            failure("Failed to set Aria network interface priority")

    if interfaces[-1] != ARIA_INTERFACE:
        remediate(
            "Aria network interface priority is too high, this may interrupt other network connectivity while streaming",
            "lower the priority of the Aria network interface",
            remediation,
        )

    success("Aria network interface priority ok")


def main():
    parser = argparse.ArgumentParser(description="Project Aria Client SDK doctor.")
    parser.add_argument(
        "-v", "--verbose", help="increase output verbosity", action="store_true"
    )
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    check_glib()
    check_python_version()
    check_macos_network()

    for configuration_file in REQUIRED_CONFIGURATION_FILES:
        configuration_file.ensure()


if __name__ == "__main__":
    main()

==> projectaria_client_sdk-1.0.3-cp39-cp39-macosx_10_16_x86_64.whl: projectaria_client_sdk-1.0.3.data/purelib/aria/doctor.py <==
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import hashlib
import logging
import os
import platform
import re
import stat
import subprocess
import sys

import tempfile
from dataclasses import dataclass
from typing import Callable, List, Optional

from termcolor import colored


def running_as_root() -> bool:
    return os.getuid() == 0


def run_command(cmd: List[str]) -> subprocess.CompletedProcess:
    logging.debug("Running: %s", " ".join(cmd))
    return subprocess.run(cmd, text=True, capture_output=True)


def failure(message: str):
    print(f"[ fail] {message}")
    exit(1)


def success(message: str):
    pill = colored("[ pass]", "white", "on_green", attrs=["bold"])
    message = colored(message, "green")
    print(pill, message)


def warning(message: str):
    pill = colored("[ warn]", "black", "on_yellow", attrs=["bold"])
    message = colored(message, "yellow")
    print(pill, message)


def action(message: str):
    print(f"    [+] {message}")


@dataclass
class Command:
    description: str
    cmd: List[str]


@dataclass
class File:
    description: str
    platforms: List[str]
    name: str
    path: str
    contents: str
    hash_suffix: bool = False
    permissions: Optional[int] = None
    owner: Optional[str] = None
    post_command: Optional[Command] = None

    def hash(self) -> str:
        return hashlib.sha1(self.contents.encode("utf8")).hexdigest()

    def file_path(self) -> str:
        if self.hash_suffix:
            name = f"{self.name}-{self.hash()}"
        else:
            name = self.name
        return os.path.join(self.path, name)

    def _read(self) -> str:
        with open(self.file_path(), "r") as f:
            return f.read()

    def _is_correct(self) -> bool:
        filepath = self.file_path()
        if not os.path.exists(filepath):
            return False
        # If we hash the file, no need to read contents
        if self.hash_suffix:
            return True
        if self.permissions is not None:
            st = os.stat(filepath)
            observed_permissions = stat.S_IMODE(st[stat.ST_MODE])
            if observed_permissions != self.permissions:
                logging.debug(
                    "Expected mode %s, got %s",
                    oct(self.permissions),
                    oct(observed_permissions),
                )
                return False
        try:
            d = self._read()
            if d != self.contents:
                logging.debug("File `%s` contents are outdated", filepath)
                return False
        except PermissionError:
            warning(f"Could not check {self.description} file, please re-run as sudo")
        return True

    def ensure(
        self,
    ):
        if platform.system() not in self.platforms:
            return

        filepath = self.file_path()

        def remediation():
            if not os.path.exists(self.path):
                failure(
                    f"Path `{self.path}` not found, please use a supported Linux distribution"
                )

            action(f"Writing {self.description} at {filepath}")

            if self.owner == "root":
                # Remove any files with outdated hashes
                if self.hash_suffix:
                    for file in os.listdir(self.path):
                        if os.path.isfile(
                            os.path.join(self.path, file)
                        ) and file.startswith(self.name + "-"):
                            action(f"Removing old {self.description} file {file}")
                            run_command(["sudo", "rm", os.path.join(self.path, file)])
                # Write file and copy to correct location
                with tempfile.NamedTemporaryFile("w") as f:
                    f.write(self.contents)
                    f.flush()
                    run_command(["sudo", "cp", f.name, filepath])
                    if self.permissions is not None:
                        run_command(
                            ["sudo", "chmod", oct(self.permissions)[2:], filepath]
                        )
            else:
                with open(filepath, "w") as f:
                    f.write(self.contents)
                if self.permissions is not None:
                    os.chmod(filepath, self.permissions)

            if self.post_command is not None:
                action(self.post_command.description)
                run_command(self.post_command.cmd)

        if not self._is_correct():
            remediate(
                f"{self.description} is not configured correctly.",
                "update the configuration",
                remediation,
            )

        success(f"{self.description} ok")


REQUIRED_CONFIGURATION_FILES = [
    File(
        description="Aria udev rules",
        platforms=["Linux"],
        name="52-aria.rules",
        path="/etc/udev/rules.d",
        owner="root",
        permissions=0o644,
        post_command=Command(
            description="Reloading udev rules",
            cmd=["sudo", "udevadm", "control", "—reload-rules"],
        ),
        contents="""
SUBSYSTEM=="usb", ATTRS{idVendor}=="2833", MODE="0666"
SUBSYSTEM=="net", ACTION=="add", ATTRS{idVendor}=="2833" ATTRS{idProduct}=="0300", NAME="aria"
""",
    ),
    File(
        description="Aria network manager connection",
        platforms=["Linux"],
        name="aria",
        path="/etc/NetworkManager/system-connections",
        permissions=0o600,
        hash_suffix=True,
        owner="root",
        post_command=Command(
            description="Reloading network manager connections",
            cmd=["sudo", "nmcli", "connection", "reload"],
        ),
        contents="""
[connection]
id=Aria
interface-name=aria
autoconnect=true
type=ethernet

[ipv4]
method=auto
never-default=true

[ipv6]
method=auto
never-default=true
""",
    ),
]


def remediate(message: str, action: str, remediation: Callable):
    print(f"[error] {message}")
    input_message = f"        Would you like to {action}? ([y]/n): "
    ans = input(input_message)
    if ans == "" or ans == "y":
        try:
            remediation()
        except Exception as e:
            logging.debug("Exception: %s", e)
            failure(
                f"Failed to {action}. Please check https://facebookresearch.github.io/projectaria_tools/docs/ARK/sdk/troubleshoot"
            )

    else:
        exit(1)


def check_glib() -> bool:
    FAILED_TO_VALIDATE = "Failed validate glibc version, please check you are using a supported Linux distribution"

    if platform.system() != "Linux":
        return True

    output = run_command(["ldd", "--version", "ldd"])

    if output.returncode != 0:
        failure(FAILED_TO_VALIDATE)

    m = re.search(r"ldd .+ (\d+).(\d+)", output.stdout)

    if not m:
        failure(FAILED_TO_VALIDATE)

    ldd_major = int(m.group(1))
    ldd_minor = int(m.group(2))

    if ldd_major != 2 or ldd_minor < 34:
        failure(
            f"Your system glibc ({ldd_major}.{ldd_minor}) is too old, at least v2.34 is required. Please use a more recent Linux distribution"
        )

    success("glibc version ok")


def check_python_version():
    ver = sys.version_info
    if ver.minor == 8 and ver.micro < 10:
        failure(
            "Python versions below 3.8.10 are not supported. Please update your python version."
        )
    success("Python version ok")


def check_macos_network():
    """
    On MacOS, check that the `Aria` network service is last in the service order.
    This ensures that Aria USB streaming does not prevent internet access
    via other services (e.g. Wi-Fi or ethernet)
    """

    if platform.system() != "Darwin":
        return

    response = run_command(["networksetup", "-listnetworkserviceorder"])

    if response.returncode != 0:
        failure("Failed to list network interfaces.")

    interfaces = [
        match.group(1)
        for match in re.finditer(r"^\(\d+\) (.+)$", response.stdout, flags=re.MULTILINE)
    ]

    ARIA_INTERFACE = "Aria"

    if ARIA_INTERFACE not in interfaces:
        warning(
            "No Aria network interface found, this will be created after you first start streaming. "
            "If you lose internet access when starting streaming over USB, please run `aria-doctor` "
            "again to restore connectivity."
        )
        return

    def remediation():
        # Move the Aria interface to lowest priority
        interfaces.remove(ARIA_INTERFACE)
        interfaces.append(ARIA_INTERFACE)

        response = run_command(
            ["sudo", "networksetup", "-ordernetworkservices"] + interfaces
        )

        if response.returncode != 0:
            logging.debug(
                "networksetup failed: %i, %d", response.returncode, response.stdout
            )
            failure("Failed to set Aria network interface priority")

    if interfaces[-1] != ARIA_INTERFACE:
        remediate(
            "Aria network interface priority is too high, this may interrupt other network connectivity while streaming",
            "lower the priority of the Aria network interface",
            remediation,
        )

    success("Aria network interface priority ok")


def main():
    parser = argparse.ArgumentParser(description="Project Aria Client SDK doctor.")
    parser.add_argument(
        "-v", "--verbose", help="increase output verbosity", action="store_true"
    )
    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)

    check_glib()
    check_python_version()
    check_macos_network()

    for configuration_file in REQUIRED_CONFIGURATION_FILES:
        configuration_file.ensure()


if __name__ == "__main__":
    main()
